// Planetary gear bearing (customizable)

//co ma byd renderowane:
ring = 1;
sun = 1;
planets =0;
logo=1;

// outer diameter of ring
D=35;
Dext=1;
// thickness
T=10;
// clearance
tol=0.2;
number_of_planets=5;
number_of_teeth_on_planets=7;
approximate_number_of_teeth_on_sun=9;
// pressure angle
P=45;//[30:60]
// number of teeth to twist across
nTwist=1;
// width of hexagonal hole.  For hex safts.
w=4.7;
// Shaft radius.  For round saft with a flat.
sd=6;
// Shaft thinkness at flat  For round saft with a flat..
st=5;
// Shaft type: 0=hex, 1=round w/flat.
stype=2;
// Planet axile diameter.
pad=4;

DR=0.5*1;// maximum depth ratio of teeth

Rus=2; //srednica ucha zewn
Ruz=4; //srednica ucha wewn

m=round(number_of_planets);
np=round(number_of_teeth_on_planets);
ns1=approximate_number_of_teeth_on_sun;
k1=round(2/m*(ns1+np));
k= k1*m%2!=0 ? k1+1 : k1;
ns=k*m/2-np;
echo(ns);
nr=ns+2*np;
pitchD=0.9*D/(1+min(PI/(2*nr*tan(P)),PI*DR/nr));
pitch=pitchD*PI/nr;
echo(pitch);
helix_angle=atan(2*nTwist*pitch/T);
echo(helix_angle);

phi=$t*360/m;



union() {
        planetarygear();
    if (logo==1) monogram(slices=100, ext=0.3);    
}


module planetarygear() {
    if(ring==1){
        difference(){ // This makes the outer ring.
            union() {
                    cylinder(r=D/2+Dext,h=T,center=true,$fn=100);
                translate([D/2+Ruz+Dext-Rus,0,-T/3]){
                    cylinder(r=Ruz,h=T/3,center=true,$fn=100);
                }
            }
            union(){
                    translate([D/2+Dext+Ruz-Rus,0,0]){
                        cylinder(r=Rus,h=T*1.2,center=true,$fn=100);
                    }  
                herringbone(nr,pitch,P,DR,-tol,helix_angle,T+0.2);
            }
            

        }
       
    }
    if (sun == 1) {
	rotate([0,0,(np+1)*180/ns+phi*(ns+np)*2/ns])
	difference(){ // This makes the sun.
		mirror([0,1,0])
			herringbone(ns,pitch,P,DR,tol,helix_angle,T);
		if(stype==0)
			cylinder(r=w/sqrt(3),h=T+1,center=true,$fn=6);
		else if(stype==1)
			difference(){ // This makes a hole for a flat shaft.
				cylinder(r=sd/2,h=T+1,center=true,$fn=100);
				translate([st-(sd/2),-(sd/2),-(T+1)/2])
					cube([sd, sd, T+1]);	// This is the flat in the shaft hole.
			}
        cylinder(r=w/sqrt(3),h=T+1,center=true,$fn=6);

	}
}
    if (planets==1) {
	// Now make the planets.
	for(i=[1:m])
		rotate([0,0,i*360/m+phi])
			translate([pitchD/2*(ns+np)/nr,0,0])
				rotate([0,0,i*ns/m*360/np-phi*(ns+np)/np-phi])
					difference(){ // make an axial hole in each planet.
						herringbone(np,pitch,P,DR,tol,helix_angle,T);
						cylinder(r=pad/2,h=T+1,center=true,$fn=100);	// This is the hole.
					}
    }
}


// Makes the teeth on the inside of the outer ring.
module monogram(slices=20, ext=0.1) {

radius = D/2+Dext-1; 
height = T*0.9; 
text_depth = (ext+1)*6; 
circumference = 2 * 3.14159 * radius; 
slice_width = circumference / slices; 
    rotate ([0,0,-80])
    union () { 
        for (i = [0:1:slices]) { 
            rotate ([0,0,i*(360/slices)]) translate ([0,-radius,0]) intersection () { 
                translate ([-slice_width/2 - (i*slice_width) ,0 ,0]) rotate ([90,0,0]) 
                scale(0.02*height) translate([-150,403,0]) linear_extrude(height = text_depth) import (file = "reprapypl_napis2.dxf");
                cube ([slice_width+0.1, text_depth+0.1, height], true); 
            } 
        } 
    } 

} 

module herringbone(
	number_of_teeth=15,
	circular_pitch=10,
	pressure_angle=28,
	depth_ratio=1,
	clearance=0,
	helix_angle=0,
	gear_thickness=5){
union(){
	gear(number_of_teeth,
		circular_pitch,
		pressure_angle,
		depth_ratio,
		clearance,
		helix_angle,
		gear_thickness/2);
	mirror([0,0,1])
		gear(number_of_teeth,
			circular_pitch,
			pressure_angle,
			depth_ratio,
			clearance,
			helix_angle,
			gear_thickness/2);
}}

module gear (
	number_of_teeth=15,
	circular_pitch=10,
	pressure_angle=28,
	depth_ratio=1,
	clearance=0,
	helix_angle=0,
	gear_thickness=5,
	flat=false){
pitch_radius = number_of_teeth*circular_pitch/(2*PI);
twist=tan(helix_angle)*gear_thickness/pitch_radius*180/PI;

flat_extrude(h=gear_thickness,twist=twist,flat=flat)
	gear2D (
		number_of_teeth,
		circular_pitch,
		pressure_angle,
		depth_ratio,
		clearance);
}

module flat_extrude(h,twist,flat){
	if(flat==false)
		linear_extrude(height=h,twist=twist,slices=twist/6)children(0);
	else
		children(0);
}

module gear2D (
	number_of_teeth,
	circular_pitch,
	pressure_angle,
	depth_ratio,
	clearance){
pitch_radius = number_of_teeth*circular_pitch/(2*PI);
base_radius = pitch_radius*cos(pressure_angle);
depth=circular_pitch/(2*tan(pressure_angle));
outer_radius = clearance<0 ? pitch_radius+depth/2-clearance : pitch_radius+depth/2;
root_radius1 = pitch_radius-depth/2-clearance/2;
root_radius = (clearance<0 && root_radius1<base_radius) ? base_radius : root_radius1;
backlash_angle = clearance/(pitch_radius*cos(pressure_angle)) * 180 / PI;
half_thick_angle = 90/number_of_teeth - backlash_angle/2;
pitch_point = involute (base_radius, involute_intersect_angle (base_radius, pitch_radius));
pitch_angle = atan2 (pitch_point[1], pitch_point[0]);
min_radius = max (base_radius,root_radius);

intersection(){
	rotate(90/number_of_teeth)
		circle($fn=number_of_teeth*3,r=pitch_radius+depth_ratio*circular_pitch/2-clearance/2);
	union(){
		rotate(90/number_of_teeth)
			circle($fn=number_of_teeth*2,r=max(root_radius,pitch_radius-depth_ratio*circular_pitch/2-clearance/2));
		for (i = [1:number_of_teeth])rotate(i*360/number_of_teeth){
			halftooth (
				pitch_angle,
				base_radius,
				min_radius,
				outer_radius,
				half_thick_angle);		
			mirror([0,1])halftooth (
				pitch_angle,
				base_radius,
				min_radius,
				outer_radius,
				half_thick_angle);
		}
	}
}}

module halftooth (
	pitch_angle,
	base_radius,
	min_radius,
	outer_radius,
	half_thick_angle){
index=[0,1,2,3,4,5,6,7,8,9];
start_angle = max(involute_intersect_angle (base_radius, min_radius)-9,0);
stop_angle = involute_intersect_angle (base_radius, outer_radius);
angle=index*(stop_angle-start_angle)/index[len(index)-1];
p=[[0,0], // The more of these the smoother the involute shape of the teeth.
	involute(base_radius,angle[0]+start_angle),
	involute(base_radius,angle[1]+start_angle),
	involute(base_radius,angle[2]+start_angle),
	involute(base_radius,angle[3]+start_angle),
	involute(base_radius,angle[4]+start_angle),
	involute(base_radius,angle[5]+start_angle),
    involute(base_radius,angle[6]+start_angle),
	involute(base_radius,angle[7]+start_angle),
	involute(base_radius,angle[8]+start_angle),
	involute(base_radius,angle[9]+start_angle)];

difference(){
	rotate(-pitch_angle-half_thick_angle)polygon(points=p);
	square(2*outer_radius);
}}

// Mathematical Functions
//===============

// Finds the angle of the involute about the base radius at the given distance (radius) from it's center.
//source: http://www.mathhelpforum.com/math-help/geometry/136011-circle-involute-solving-y-any-given-x.html

function involute_intersect_angle (base_radius, radius) = sqrt (pow (radius/base_radius, 2) - 1) * 180 / PI;

// Calculate the involute position for a given base radius and involute angle.

function involute (base_radius, involute_angle) =
[
	base_radius*(cos (involute_angle) + involute_angle*PI/180*sin (involute_angle)),
	base_radius*(sin (involute_angle) - involute_angle*PI/180*cos (involute_angle))
];
